@@ -64,6 +64,25 @@ fastmap2 <- function(missing_default = NULL){
   map
 }
 
+#' @title Copy elements to \code{fastmap2}
+#' @param li a list or an environment
+#' @param map \code{NULL} or a \code{fastmap2} instance
+#' @return If \code{map} is not \code{NULL}, elements will be added
+#' to \code{map} and return \code{map}, otherwise create a new instance.
+#' @export
+list_to_fastmap2 <- function(li, map = NULL){
+  stopifnot2(is.null(map) || inherits(map, 'fastmap2'), msg = 'map must be either NULL or fastmap2')
+  if(is.null(map)){
+    map <- fastmap2()
+  }
+  for(nm in names(li)){
+    if(nm != ''){
+      map[[nm]] <- li[[nm]]
+    }
+  }
+  map
+}
+
 #' @title Migrate a \code{fastmap2} object to a new one
 #' @param from,to \code{fastmap2} object
 #' @param override whether to override keys in \code{to} if they exist
@@ -86,10 +105,11 @@ update_fastmap2 <- function(from, to, override = TRUE){
 #' @rdname fastmap2
 #' @export
 `[[.fastmap2` <- function(x, name){
+  name <- as.character(name)
   if( startsWith(name, '@') ){
     .subset2(x, substring(name, 2))
   }else{
-    .subset2(x, 'get')(as.character(name))
+    .subset2(x, 'get')(name)
   }
 }
 
@@ -118,9 +138,18 @@ update_fastmap2 <- function(from, to, override = TRUE){
 #' @export
 `[<-.fastmap2` <- function(x, i, j = NULL, ..., value){
   i <- unlist(c(i, j, ...))
-  stopifnot2(length(value) == length(i),
+  # instead of throwing error,
+  stopifnot2(length(value) <= 1 || length(value) == length(i),
              msg='value must be the same length as name')
-  do.call(.subset2(x, 'mset'), structure(as.list(value), names = as.character(i)))
+  if( length(value) == length(i) ){
+    .subset2(x, 'mset')(.list = structure(as.list(value), names = as.character(i)))
+  } else {
+    # set for each key
+    for(k in i){
+      .subset2(x, 'set')(as.character(k), value)
+    }
+  }
+
   x
 }
 
